/* *********************************************************************
 * This Original Work is copyright of 51 Degrees Mobile Experts Limited.
 * Copyright 2023 51 Degrees Mobile Experts Limited, Davidson House,
 * Forbury Square, Reading, Berkshire, United Kingdom RG1 3EU.
 *
 * This Original Work is licensed under the European Union Public Licence
 * (EUPL) v.1.2 and is subject to its terms as set out below.
 *
 * If a copy of the EUPL was not distributed with this file, You can obtain
 * one at https://opensource.org/licenses/EUPL-1.2.
 *
 * The 'Compatible Licences' set out in the Appendix to the EUPL (as may be
 * amended by the European Commission) shall be deemed incompatible for
 * the purposes of the Work and the provisions of the compatibility
 * clause in Article 5 of the EUPL shall not apply.
 *
 * If using the Work as, or as part of, a network application, by
 * including the attribution notice(s) required under Article 5 of the EUPL
 * in the end user terms of the application under an appropriate heading,
 * such notice(s) shall fulfill the requirements of that article.
 * ********************************************************************* */

#include "collectionKeyTypes.h"

#ifndef FIFTYONE_DEGREES_MEMORY_ONLY
uint32_t fiftyoneDegreesThrowUnsupportedStoredValueType(
    const void * const initial,
    fiftyoneDegreesException * const exception) {
#	ifdef _MSC_VER
    UNREFERENCED_PARAMETER(initial);
#	endif
    FIFTYONE_DEGREES_EXCEPTION_SET(FIFTYONE_DEGREES_STATUS_UNSUPPORTED_STORED_VALUE_TYPE);
    return 0;
}
#endif

const fiftyoneDegreesCollectionKeyType *fiftyoneDegreesGetCollectionKeyTypeForStoredValueType(
    const fiftyoneDegreesPropertyValueType storedValueType,
    fiftyoneDegreesException * const exception) {
#	ifdef _MSC_VER
    UNREFERENCED_PARAMETER(exception);
#	endif

    switch (storedValueType) {
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_STRING:
            return CollectionKeyType_String;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_INTEGER:
            return CollectionKeyType_Integer;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_AZIMUTH:
            return CollectionKeyType_Azimuth;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_DECLINATION:
            return CollectionKeyType_Declination;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_SINGLE_PRECISION_FLOAT:
            return CollectionKeyType_Float;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_SINGLE_BYTE:
            return CollectionKeyType_Byte;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_IP_ADDRESS:
            return CollectionKeyType_IPAddress;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_WKB_R:
            return CollectionKeyType_WKB_R;
        case FIFTYONE_DEGREES_PROPERTY_VALUE_TYPE_WKB:
            return CollectionKeyType_WKB;
        default: {
            return CollectionKeyType_Unsupported;
        }
    }
}
