/* *********************************************************************
* This Original Work is copyright of 51 Degrees Mobile Experts Limited.
 * Copyright 2023 51 Degrees Mobile Experts Limited, Davidson House,
 * Forbury Square, Reading, Berkshire, United Kingdom RG1 3EU.
 *
 * This Original Work is licensed under the European Union Public Licence
 * (EUPL) v.1.2 and is subject to its terms as set out below.
 *
 * If a copy of the EUPL was not distributed with this file, You can obtain
 * one at https://opensource.org/licenses/EUPL-1.2.
 *
 * The 'Compatible Licences' set out in the Appendix to the EUPL (as may be
 * amended by the European Commission) shall be deemed incompatible for
 * the purposes of the Work and the provisions of the compatibility
 * clause in Article 5 of the EUPL shall not apply.
 *
 * If using the Work as, or as part of, a network application, by
 * including the attribution notice(s) required under Article 5 of the EUPL
 * in the end user terms of the application under an appropriate heading,
 * such notice(s) shall fulfill the requirements of that article.
 * ********************************************************************* */

#ifndef TESTUTILS_POINTERS_HPP
#define TESTUTILS_POINTERS_HPP

#include "../fiftyone.h"
#include <memory>

static void freeCollection(fiftyoneDegreesCollection * const ptr) {
    if (ptr) {
        ptr->freeCollection(ptr);
    }
}
using CollectionPtr = std::unique_ptr<fiftyoneDegreesCollection, decltype(&freeCollection)>;


static void releaseItem(Item * const item) {
    if (item) {
        COLLECTION_RELEASE(item->collection, item);
    }
}
using ItemPtr = std::unique_ptr<Item, decltype(&releaseItem)>;

class ItemBox {
    Item item = {};
public:
    ItemBox() {
        DataReset(&item.data);
    }
    ~ItemBox() {
        if (item.collection) {
            COLLECTION_RELEASE(item.collection, &item);
        }
    }
    Item *operator*() { return &item; }
    Item *operator->() { return &item; }
};

#endif //TESTUTILS_POINTERS_HPP
